using System;
using System.Collections.Generic;
using Dakota.Data;
using Microsoft.Xna.Framework;
using Nez;

namespace Dakota.Renderables
{
    public class VectorText : RenderableComponent
    {
        private const char Unprintable = '~';

        private static Dictionary<char, Vector2?[]> _font = new Dictionary<char, Vector2?[]>();
        public int Size;
        public string Text;
        public float Thickness = 1.0f;

        static VectorText()
        {
            // each entry corresponds to the vectors for a given glyph
            // lines are drawn from point to point the way one would if writing
            // a null will "lift the pen" so to speak, starting a new set of
            // lines not connected to the prior set
            _font[Unprintable] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(1, 1),
                null,
                new Vector2(0, 1),
                new Vector2(1, 0)
            };
            _font['A'] = new Vector2?[]
            {
                new Vector2(0, 1),
                new Vector2(0.5f, 0),
                new Vector2(1, 1),
                null,
                new Vector2(0.2f, 0.5f),
                new Vector2(0.7f, 0.5f)
            };
            _font['B'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 0.8f),
                new Vector2(1, 0.6f),
                new Vector2(0, 0.5f),
                new Vector2(1, 0.4f),
                new Vector2(1, 0.2f),
                new Vector2(0, 0)
            };
            _font['C'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1)
            };
            _font['D'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(0.5f, 1),
                new Vector2(1, 0.6f),
                new Vector2(1, 0.4f),
                new Vector2(0.5f, 0),
                new Vector2(0, 0)
            };
            _font['E'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                null,
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f)
            };
            _font['F'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                null,
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f)
            };
            _font['G'] = new Vector2?[]
            {
                new Vector2(1, 0.2f),
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0.7f),
                new Vector2(0.5f, 0.7f)
            };
            _font['H'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                null,
                new Vector2(1, 0),
                new Vector2(1, 1),
                null,
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f)
            };
            _font['I'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                null,
                new Vector2(0, 1),
                new Vector2(1, 1),
                null,
                new Vector2(0.5f, 0),
                new Vector2(0.5f, 1)
            };
            _font['J'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(1, 1),
                new Vector2(0.5f, 1),
                new Vector2(0, 0.7f)
            };
            _font['K'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0.5f),
                new Vector2(1, 1),
                null,
                new Vector2(0, 0),
                new Vector2(0, 1)
            };
            _font['L'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1)
            };
            _font['M'] = new Vector2?[]
            {
                new Vector2(0, 1),
                new Vector2(0, 0),
                new Vector2(0.5f, 0.2f),
                new Vector2(1, 0),
                new Vector2(1, 1)
            };
            _font['N'] = new Vector2?[]
            {
                new Vector2(0, 1),
                new Vector2(0, 0),
                new Vector2(1, 1),
                new Vector2(1, 0)
            };
            _font['O'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0)
            };
            _font['P'] = new Vector2?[]
            {
                new Vector2(0, 1),
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(1, 0.4f),
                new Vector2(0, 0.4f)
            };
            _font['Q'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0),
                null,
                new Vector2(1, 1),
                new Vector2(0.5f, 0.6f)
            };
            _font['R'] = new Vector2?[]
            {
                new Vector2(0, 1),
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(1, 0.4f),
                new Vector2(0, 0.4f),
                new Vector2(1, 1)
            };
            _font['S'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f),
                new Vector2(1, 1),
                new Vector2(0, 1)
            };
            _font['T'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                null,
                new Vector2(0.5f, 0),
                new Vector2(0.5f, 1)
            };
            _font['U'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0)
            };
            _font['V'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0.5f, 1),
                new Vector2(1, 0)
            };
            _font['W'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(0.5f, 0.7f),
                new Vector2(1, 1),
                new Vector2(1, 0)
            };
            _font['X'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 1),
                null,
                new Vector2(1, 0),
                new Vector2(0, 1)
            };
            _font['Y'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0.5f, 0.4f),
                new Vector2(1, 0),
                null,
                new Vector2(0.5f, 0.4f),
                new Vector2(0.5f, 1)
            };
            _font['Z'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(0, 1),
                new Vector2(1, 1)
            };
            _font['0'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0),
                new Vector2(0, 1)
            };
            _font['1'] = new Vector2?[]
            {
                new Vector2(0.5f, 1),
                new Vector2(0.5f, 0),
                new Vector2(0.3f, 0.2f)
            };
            _font['2'] = new Vector2?[]
            {
                new Vector2(0, 0.2f),
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(0, 1),
                new Vector2(1, 1)
            };
            _font['3'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f),
                new Vector2(1, 1),
                new Vector2(0, 1)
            };
            _font['4'] = new Vector2?[]
            {
                new Vector2(0.8f, 1),
                new Vector2(0.8f, 0),
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f)
            };
            _font['5'] = new Vector2?[]
            {
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 0.5f),
                new Vector2(1, 0.5f),
                new Vector2(0, 1)
            };
            _font['6'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(1, 0.6f),
                new Vector2(0, 0.6f)
            };
            _font['7'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(0, 1)
            };
            _font['8'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(0, 1),
                new Vector2(1, 1),
                new Vector2(0, 0)
            };
            _font['9'] = new Vector2?[]
            {
                new Vector2(1, 1),
                new Vector2(1, 0),
                new Vector2(0, 0),
                new Vector2(0, 0.4f),
                new Vector2(1, 0.4f)
            };
            _font[' '] = new Vector2?[] { };
            _font['.'] = new Vector2?[]
            {
                new Vector2(0.5f, 0.9f),
                new Vector2(0.6f, 0.9f),
                new Vector2(0.6f, 1),
                new Vector2(0.5f, 1),
                new Vector2(0.5f, 0.9f)
            };
            _font['!'] = new Vector2?[]
            {
                new Vector2(0.55f, 0),
                new Vector2(0.55f, 0.8f),
                null,
                new Vector2(0.5f, 0.9f),
                new Vector2(0.6f, 0.9f),
                new Vector2(0.6f, 1),
                new Vector2(0.5f, 1),
                new Vector2(0.5f, 0.9f)
            };
            _font['?'] = new Vector2?[]
            {
                new Vector2(0, 0),
                new Vector2(1, 0),
                new Vector2(1, 0.4f),
                new Vector2(0.55f, 0.4f),
                new Vector2(0.55f, 0.8f),
                null,
                new Vector2(0.5f, 0.9f),
                new Vector2(0.6f, 0.9f),
                new Vector2(0.6f, 1),
                new Vector2(0.5f, 1),
                new Vector2(0.5f, 0.9f)
            };
            _font[','] = new Vector2?[]
            {
                new Vector2(0.7f, 0.6f),
                new Vector2(0.6f, 1)
            };
            _font['\''] = new Vector2?[]
            {
                new Vector2(0.7f, 0),
                new Vector2(0.6f, 0.3f)
            };
            _font[':'] = new Vector2?[]
            {
                new Vector2(0.5f, 0.9f),
                new Vector2(0.6f, 0.9f),
                new Vector2(0.6f, 1),
                new Vector2(0.5f, 1),
                new Vector2(0.5f, 0.9f),
                null,
                new Vector2(0.5f, 0),
                new Vector2(0.6f, 0),
                new Vector2(0.6f, 0.1f),
                new Vector2(0.5f, 0.1f),
                new Vector2(0.5f, 0)
            };
            _font['-'] = new Vector2?[]
            {
                new Vector2(0.2f, 0.5f),
                new Vector2(0.8f, 0.5f)
            };
            _font['+'] = new Vector2?[]
            {
                new Vector2(0.2f, 0.5f),
                new Vector2(0.8f, 0.5f),
                null,
                new Vector2(0.5f, 0.2f),
                new Vector2(0.5f, 0.8f)
            };
        }

        public VectorText(string text, int size, Color color)
        {
            Text = text;
            Size = size;
            Color = color;
            RenderLayer = GameConstants.UiRenderLayerFg;
        }

        // approximate
        public override float Width => Size * Text.Length;
        public override float Height => Size;

        public override void Render(Batcher batcher, Camera camera)
        {
            var spacing = 5;
            var curOffset = new Vector2(-Size, 0);

            foreach (var ch in Text)
            {
                Vector2?[] letter;
                if (ch == '\n')
                {
                    curOffset.X = -Size;
                    curOffset.Y += Size + spacing;
                    continue;
                }
                else if (_font.ContainsKey(ch))
                {
                    letter = _font[ch];
                    curOffset.X += Size + spacing;
                }
                else
                {
                    letter = _font[Unprintable];
                    curOffset.X += Size + spacing;
                    Console.WriteLine($"UNPRINTABLE: {ch}");
                }

                for (var i = 0; i < letter.Length - 1; i++)
                    if (letter[i] != null && letter[i + 1] != null)
                        batcher.DrawLine(
                            (Vector2) letter[i] * Size + curOffset + Entity.Position + LocalOffset,
                            (Vector2) letter[i + 1] * Size + curOffset + Entity.Position + LocalOffset,
                            Color,
                            Thickness
                        );
            }
        }
    }
}