using Dakota.Data;
using Dakota.Renderables;
using Microsoft.Xna.Framework;
using Nez;

namespace Dakota.Space
{
    public class SpaceHud : RenderableComponent, IUpdatable
    {
        private const int FrameSize = 30;
        private VectorText _acceleration;
        private VectorText _primary;
        private VectorText _spaceCoords;
        private VectorText _velocity;
        private VectorText _objective;

        private VectorText _zoneCoords;
        public override float Width => 280;
        public override float Height => 720;

        public void Update()
        {
            var scene = (SpaceScene) Entity.Scene;
            var obj = scene.Player.GetComponent<SpaceObject>();
            _primary.Text = scene.Primary.Name;
            _objective.Text = scene.CurrentMission.GetMissionStatus();
            _zoneCoords.Text = $"Z: {scene.CurrentZone.UniversalX}  {scene.CurrentZone.UniversalY}";
            _spaceCoords.Text = $"P: {scene.Player.Position.X:0}  {scene.Player.Position.Y:0}";
            _velocity.Text = $"V: {obj.Velocity.X:0}  {obj.Velocity.Y:0}";
            _acceleration.Text = $"A: {obj.Acceleration.X:0}  {obj.Acceleration.Y:0}";
        }

        public override void OnAddedToEntity()
        {
            RenderLayer = GameConstants.UiRenderLayerBg;

            var map = Entity.AddComponent(new SpaceHudMap());
            map.LocalOffset = new Vector2(40, 500);
            map.RenderLayer = GameConstants.UiRenderLayerBg;

            var line = Entity.AddComponent(new VectorText("PRIMARY", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 40);
            _primary = line = Entity.AddComponent(new VectorText("", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 60);
            line = Entity.AddComponent(new VectorText("SECONDARY", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 100);
            line = Entity.AddComponent(new VectorText("SMART BOMB", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 120);
            line = Entity.AddComponent(new VectorText("WARP CORE", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 200);
            line = Entity.AddComponent(new VectorText("DISABLED", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 220);
            line = Entity.AddComponent(new VectorText("OBJECTIVE", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 250);
            _objective = line = Entity.AddComponent(new VectorText("", 6, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 270);

            line = Entity.AddComponent(new VectorText("COORDINATES", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 350);
            _zoneCoords = line = Entity.AddComponent(new VectorText("", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 370);
            _spaceCoords = line = Entity.AddComponent(new VectorText("", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 390);
            _velocity = line = Entity.AddComponent(new VectorText("", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 410);
            _acceleration = line = Entity.AddComponent(new VectorText("", 12, GameConstants.SpaceHudFontColor));
            line.LocalOffset = new Vector2(40, 430);
        }

        public override void Render(Batcher batcher, Camera camera)
        {
            batcher.DrawRect(Entity.Position, Width, Height, GameConstants.ColorBlack);
            batcher.DrawHollowRect(Entity.Position, Width, Height, GameConstants.SpaceUiColor);
            batcher.DrawHollowRect(
                Entity.Position + new Vector2(FrameSize, FrameSize),
                Width - FrameSize * 2, Height - FrameSize * 2,
                GameConstants.SpaceUiColor);
            batcher.DrawLine(
                Entity.Position,
                Entity.Position + new Vector2(FrameSize, FrameSize),
                GameConstants.SpaceUiColor);
            batcher.DrawLine(
                Entity.Position + new Vector2(0, Height),
                Entity.Position + new Vector2(FrameSize, Height - FrameSize),
                GameConstants.SpaceUiColor);
            batcher.DrawLine(
                Entity.Position + new Vector2(Width, Height),
                Entity.Position + new Vector2(Width - FrameSize, Height - FrameSize),
                GameConstants.SpaceUiColor);
            batcher.DrawLine(
                Entity.Position + new Vector2(Width, 0),
                Entity.Position + new Vector2(Width - FrameSize, FrameSize),
                GameConstants.SpaceUiColor);
        }
    }
}