using System;
using Dakota.Utils;
using Microsoft.Xna.Framework;
using Nez;

namespace Dakota.Space
{
    public class SpaceObject : Component, IUpdatable
    {
        private Mover _mover;
        public float TurnVelocity;
        public Vector2 Velocity;
        public Vector2 Acceleration;

        public SpaceObject()
        {
            Velocity = new Vector2(0, 0);
            Acceleration = new Vector2(0, 0);
        }

        public void Update()
        {
            var scene = (SpaceScene) Entity.Scene;

            Entity.Rotation += Time.DeltaTime * TurnVelocity;
            Velocity += Time.DeltaTime * Acceleration;
            _mover.Move(
                Time.DeltaTime * Velocity + 0.5f * Time.DeltaTime * Time.DeltaTime * Acceleration,
                out _
            );

            // enforce zone boundaries
            var newPos = Entity.Position;
            if (Entity.Position.X < 0)
            {
                if (scene.CurrentZone.IsCurvedX)
                    newPos.X += scene.CurrentZone.Width;
                else if (Entity.Name == "player")
                    newPos = scene.ZoneShift(-1, 0);
            }
            else if (Entity.Position.X > scene.CurrentZone.Width)
            {
                if (scene.CurrentZone.IsCurvedX)
                    newPos.X -= scene.CurrentZone.Width;
                else if (Entity.Name == "player")
                    newPos = scene.ZoneShift(1, 0);
            }

            if (Entity.Position.Y < 0)
            {
                if (scene.CurrentZone.IsCurvedY)
                    newPos.Y += scene.CurrentZone.Height;
                else if (Entity.Name == "player")
                    newPos = scene.ZoneShift(0, -1);
            }
            else if (Entity.Position.Y > scene.CurrentZone.Height)
            {
                if (scene.CurrentZone.IsCurvedY)
                    newPos.Y -= scene.CurrentZone.Height;
                else if (Entity.Name == "player")
                    newPos = scene.ZoneShift(0, 1);
            }

            Entity.Position = newPos;
        }

        public override void OnAddedToEntity()
        {
            _mover = Entity.AddComponent(new Mover());
        }

        public void SetAcceleration(float accel)
        {
            Acceleration = accel * DakotaUtils.VecFromAngle(Entity.Rotation);
        }
    }
}